"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_extra_1 = __importDefault(require("fs-extra"));
const aws_sdk_1 = __importDefault(require("aws-sdk"));
const mime_types_1 = __importDefault(require("mime-types"));
const path_1 = __importDefault(require("path"));
const getMimeType = (filePath) => mime_types_1.default.lookup(path_1.default.basename(filePath)) || 'application/octet-stream';
exports.default = async ({ bucketName, credentials }) => {
    let s3 = new aws_sdk_1.default.S3({ ...credentials });
    try {
        const { Status } = await s3
            .getBucketAccelerateConfiguration({
            Bucket: bucketName,
        })
            .promise();
        if (Status === 'Enabled') {
            s3 = new aws_sdk_1.default.S3({ ...credentials, useAccelerateEndpoint: true });
        }
    }
    catch (err) {
        console.warn(`Checking for bucket acceleration failed, falling back to non-accelerated S3 client. Err: ${err.message}`);
    }
    return {
        uploadFile: async (options) => {
            const { filePath, cacheControl, s3Key } = options;
            const fileBody = await fs_extra_1.default.readFile(filePath);
            return s3
                .upload({
                Bucket: bucketName,
                Key: s3Key || filePath,
                Body: fileBody,
                ContentType: getMimeType(filePath),
                CacheControl: cacheControl || undefined,
            })
                .promise();
        },
    };
};
